/* $Id: dupefilter.cpp 464 2011-03-03 21:23:13Z oh2gve $
 * 
 * Copyright 2009, 2010, 2011 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


// Own stuff.
#include "dupefilter.h"
#include "aprspacket.h"

// std stuff.
#include <list>
using std::list;
#include <map>
using std::multimap;
using std::pair;
using std::make_pair;
#include <string>
using std::string;
#include <ctime>

// wx stuff.
#include <wx/thread.h>



DupeFilter::DupeFilter(time_t const& dupetime):
wxThread(wxTHREAD_JOINABLE), mutex_(), keepgoing_(true), storage_(), dupetime_(dupetime)
{
}



DupeFilter::~DupeFilter()
{
	while ( !storage_.empty() )
	{
		delete storage_.begin()->second;
		storage_.erase(storage_.begin());
	}
}



wxThread::ExitCode DupeFilter::Entry()
{
	// Main loop
	while ( keepGoing() )
	{
		// Main loop is run once in second.
		Sleep(1000);
		
		// Check for too old packets in storage, remove if found.
		mutex_.Lock();
		multimap<string const, AprsPacket const*>::iterator i = storage_.begin();
		while ( i != storage_.end() )
		{
		        multimap<string const, AprsPacket const*>::iterator current = i++;
			if ( time(NULL) - current->second->getTimestamp() > dupetime_ )
			{
				// Too old.
				delete current->second;
				storage_.erase(current);
			}
		}
		mutex_.Unlock();
	}
	
	return NULL;
}



void DupeFilter::terminate()
{
	mutex_.Lock();
	keepgoing_ = false;
	mutex_.Unlock();
}



bool DupeFilter::isDupe(AprsPacket const* packet)
{
	mutex_.Lock();
	
	// Check if given packet is already in storage.
	pair< multimap<string const, AprsPacket const*>::iterator,
	      multimap<string const, AprsPacket const*>::iterator > packetRange = storage_.equal_range(packet->getLabel());
	if ( packetRange.second != packetRange.first )
	{
		// Check packets in the found range for exact match.
		for ( multimap<string const, AprsPacket const*>::iterator i = packetRange.first; i != packetRange.second; ++i )
		{
			if ( *i->second == *packet )
			{
				// Found it, it's a dupe.
				mutex_.Unlock();
				return true;
			}
		}
	}
	
	// Apparently not a dupe, save it in storage.
	AprsPacket const* tmp = new AprsPacket(*packet);
	storage_.insert( make_pair(packet->getLabel(), tmp) );
	
	mutex_.Unlock();
	return false;
}



bool DupeFilter::keepGoing()
{
	mutex_.Lock();
	bool retval = keepgoing_;
	mutex_.Unlock();
	return retval;
}
