/* $Id: dupefilter.h 462 2011-02-28 15:20:05Z oh2gve $
 * 
 * Copyright 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef DUPEFILTER_H
#define DUPEFILTER_H


// own stuff.
#include "aprspacket.h"

// std stuff.
#include <map>
#include <string>
#include <ctime>

// wx stuff.
#include <wx/thread.h>


/// Duplicate packet detector.
class DupeFilter : public wxThread
{
public:
		
	/// Constructor.
	DupeFilter(time_t const& dupetime);
	
	/// Destructor.
	~DupeFilter();
	
	/// Main function of the thread.
	virtual ExitCode Entry();
	
	/// Terminates the thread.
	void terminate();
	
	/// Checks if given packet is a duplicate.
	bool isDupe(AprsPacket const* packet);
	
private:
	
	/// Maintains exclusive access to thread's private data.
	wxMutex mutex_;
	
	/// Thread status flag.
	bool keepgoing_;
	
	/// Status flag checker.
	bool keepGoing();
	
	/// List of recently seen packets sorted by targets name.
	std::multimap<std::string const, AprsPacket const*> storage_;
	
	/// Amount of time to store packets in storage_.
	time_t const dupetime_;
};

#endif // DUPEFILTER_H
