/* $Id: packet.cpp 445 2011-02-21 09:17:48Z oh2gve $
 * 
 * Copyright 2011 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


// Own stuff.
#include "packet.h"

// std stuff.
#include <string>
using std::string;
#include <sstream>
using std::ostringstream;
#include <iomanip>

// wx stuff.
#include <wx/datetime.h>



Packet::Packet():
input_('-'), output_('_'), inputName_(), aprsPacket_(NULL), originalPacket_(NULL), flag_('-')
{
}



Packet::Packet(char const& input, char const& output, AprsPacket* const& data, char const& flag):
input_(input), output_(output), inputName_(), aprsPacket_(data), originalPacket_(NULL), flag_(flag)
{
}



Packet::Packet(Packet const& orig):
input_(orig.input_), output_(orig.output_), inputName_(orig.inputName_),
aprsPacket_(NULL), originalPacket_(NULL), flag_(orig.flag_)
{
	if ( orig.aprsPacket_ )
	{
		aprsPacket_ = new AprsPacket(*orig.aprsPacket_);
	}
	if ( orig.originalPacket_ )
	{
		originalPacket_ = new AprsPacket(*orig.originalPacket_);
	}
}



Packet::~Packet()
{
	if ( aprsPacket_ )
	{
		delete aprsPacket_;
	}
	if ( originalPacket_ )
	{
		delete originalPacket_;
	}
}



string Packet::toString(bool const& toLogServer) const
{
	// Print original packet.
	AprsPacket const* packet = originalPacket_;
	
	// If there's no orignal packet, use current one.
	if ( packet == NULL )
	{
		packet = aprsPacket_;
	}
	
	// If there's none, print none.
	if ( packet == NULL )
	{
		return "";
	}
	
	// Construct line.
	ostringstream output;

	// Timestamp.
	if ( toLogServer )
	{
		output << wxDateTime(packet->getTimestamp()).Format(wxT("%Y-%m-%d %H:%M:%S"), wxDateTime::UTC).ToAscii();
	}
	else
	{
		output << wxDateTime(packet->getTimestamp()).Format(wxT("%Y-%m-%d %H:%M:%S")).ToAscii();
	}
	
	// Status info.
	output << " i" << input_ << " o" << output_ << " " << flag_ << " " << std::setw(3)
	 << packet->getTnc2Packet().size() << "B ";
	
	// Actual packet.
	if ( toLogServer )
	{
		output << packet->getTnc2Packet() << "\r\n";
	}
	else
	{
		output << packet->getLogLine() << "\n";
	}
	
	return output.str();
}



void Packet::updateTimestamp()
{
	if ( aprsPacket_ == NULL )
	{
		return;
	}
	
	aprsPacket_->setTimestamp(time(NULL));
}


void Packet::setFlag(char const& flag)
{
	flag_ = flag;
}



char Packet::getInputId() const
{
	return input_;
}



void Packet::setOutputId(char const& id)
{
	output_ = id;
}



void Packet::setInputName(string const& name)
{
	inputName_ = name;
}



string Packet::getInputName() const
{
	return inputName_;
}



void Packet::setAprsPacket(AprsPacket* const& data)
{
	if ( originalPacket_ == NULL )
	{
		originalPacket_ = aprsPacket_;
	}
	else
	{
		delete aprsPacket_;
	}

	aprsPacket_ = data;
}



AprsPacket const* Packet::getAprsPacket() const
{
	return aprsPacket_;
}
