/* $Id: packet.h 445 2011-02-21 09:17:48Z oh2gve $
 * 
 * Copyright 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef PACKET_H
#define PACKET_H


// Own stuff.
#include "aprspacket.h"

// std stuff.
#include <string>

// wx stuff.
#include <wx/datetime.h>


/// One APRS packet and related data.
class Packet
{
public:
	
	/// Empty constructor.
	Packet();
	
	/// Full constructor.
	Packet(char const& input, char const& output, AprsPacket* const& packet, char const& flag);
	
	/// Copy constructor.
	Packet(Packet const& orig);
	
	/// Destructor.
	~Packet();
	
	/// Create string representation of the packet for packet logging.
	/**
	 * \param toLogServer Set to get log server compatible string, clear to get log file string.
	*/
	std::string toString(bool const& toLogServer) const;
	
	/// Update packet's time stamp to current time.
	void updateTimestamp();
	
	// Some setters and getters.

	void setFlag(char const& flag);
	char getInputId() const;
	void setOutputId(char const& id);

	void setInputName(std::string const& name);
	std::string getInputName() const;

	void setAprsPacket(AprsPacket* const& packet);
	AprsPacket const* getAprsPacket() const;
	
private:
	
	/// Number of inputting connection or u for uplink.
	char input_;
	
	/// Number of outputting connection.
	char output_;

	/// Name of inputting connection.
	std::string inputName_;
	
	/// The APRS packet which has actual APRS target as source.
	AprsPacket* aprsPacket_;
	
	/// Original packet received from hardware.
	AprsPacket* originalPacket_;
	
	/// Status flag of original packet.
	char flag_;
};


#endif // PACKET_H
