/* $Id: packetlog.h 439 2011-02-16 20:55:02Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 *
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef PACKETLOG_H
#define PACKETLOG_H

// own stuff.
#include "logfile.h"
#include "logserver.h"
#include "cfg.h"
#include "packet.h"

// std stuff.
#include <queue>

// wx stuff.
#include <wx/thread.h>
#include <wx/string.h>


/// Packet logging manager. Can be configured to log into files and/or tcp socket.
class Packetlog : public wxThread
{
public:
		
	/// Constructor.
	/**
	 * Opens files and sockets. Remember to use operator! to check how it went.
	 * \param cfg Tells us where to log.
	 * \param ownCall Callsign of this IGate.
	 */
	Packetlog(Cfg::LogCfg const& cfg, wxString const& ownCall);
	
	/// Destructor.
	/**
	 * Closes files and sockets.
	 */
	~Packetlog();
	
	/// Main function of the thread.
	virtual ExitCode Entry();
	
	/// Blocks until operator!() can be called.
	/**
	 * Call this before checking if initialization was succesfull.
	*/
	void waitForInit();

	/// Status query.
	/**
	 * Call this periodically to see how logging operation is going.
	 * \return true Everything is working as configured.
	 * \return false There's a fatal error.
	 */
	bool operator!() const;
	
	/// Thread termination.
	void terminate();
	
	/// Reopen log file.
	/**
	 * Does nothing if log file is not in use.
	 * Use operator!() to check for failure.
	 */
	void reopen();
	
	/// Places given packet into write queue.
	/**
	 * Memory occupied by the packet will be freed after write.
	 */
	void write(Packet const* packet);
	
private:
	
	/// Writes a packet to all enabled targets.
	void actualWrite(Packet const* packet);
	
	/// Read status flag in safe way.
	bool keepGoing();
	
	/// Reopens log file.
	bool cycleFile();
	
	
	/// Controls access to private data of this thread.
	wxMutex mutex_;

	/// Status flag set by constructor.
	bool initialized_;
	
	/// Status flag set by main thread.
	bool keepgoing_;
	
	/// Log file reopening signal from main.
	bool shouldReopen_;
	
	/// Our config.
	Cfg::LogCfg const& cfg_;
	
	/// Our callsign.
	wxString const ownCall_;
	
	/// Buffer of waiting packets.
	std::queue<Packet const*> buffer_;
	
	// File and/or server are set to NULL if not in use.
	
	/// Log file.
	Logfile* file_;
	
	/// Log server.
	LogServer* server_;
};

#endif // PACKETLOG_H
