/* $Id: serial.cpp 454 2011-02-27 10:30:45Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// own stuff.
#include "serial.h"
#include "rfcon.h"
#include "cfg.h"
using Cfg::ConnectionCfg;
#include "packetlog.h"
#include "dupefilter.h"
#include "serialport.h"

// std stuff.
#include <string>
using std::string;

// wx stuff.
#include <wx/log.h>
#include <wx/utils.h>


Serial::Serial(ConnectionCfg const& cfg, char const& conId, string const& owncall, Packetlog* packetlog, DupeFilter* dupefilter, wxString const& lockDir):
RFCon(cfg, conId, owncall, packetlog, dupefilter),
deviceFile_(cfg.device), device_(NULL)
{
	string lockFileName = "";
#ifndef __WXMSW__
	// First we need to find out the name of our port file without any dirs.
	string df (deviceFile_.ToAscii());
	string::size_type pos = df.find_last_of("/");
	if ( pos != string::npos )
	{
		lockFileName = lockDir.ToAscii();
		lockFileName += "/LCK.." + df.substr(pos+1);

		device_ = new Serialport(string(deviceFile_.ToAscii()), lockFileName, 1, cfg.portcfg.speed, cfg.portcfg.charlength, cfg.portcfg.parity, cfg.portcfg.stopbits, cfg.portcfg.rtscts);
#else
		device_ = new Serialport(string(deviceFile_.ToAscii()), 1, cfg.portcfg.speed, cfg.portcfg.charlength, cfg.portcfg.parity, cfg.portcfg.stopbits, cfg.portcfg.rtscts);
#endif
		// Check how it did.
		if ( !(*device_) )
		{
			wxLogError( wxT("Failed to open device. ") + wxString::FromAscii(device_->getErrorMessage().c_str()) );
			delete device_;
			device_ = NULL;
		}
		else
		{
			// Successfully opened, change our status.
			wxLogMessage(wxT(" serial(") + deviceFile_ + wxT("): device opened."));
		}
#ifndef __WXMSW__
	}
	else
	{
		wxLogError(wxT("Invalid device name \"") + deviceFile_ + wxT("\"."));
	}
#endif
}

Serial::~Serial()
{
	// Close the device.
	delete device_;
}


// PROTECTED


bool Serial::readDevice(string& data)
{
	// Check for existance.
	if ( device_ == NULL )
	{
		wxLogMessage(wxT(" serial(") + deviceFile_ + wxT("): can't read data, device is not open."));
		return false;
	}
	
	// Check status.
	if ( !(*device_) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): can't read data, ") + wxString::FromAscii(device_->getErrorMessage().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	// Try to read data.
	if ( !device_->receive(data) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): ") + wxString::FromAscii(device_->getErrorMessage().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	return true;
}

bool Serial::writeToDevice(const std::string& data)
{
	// Check for existance.
	if ( device_ == NULL )
	{
		wxLogMessage(wxT(" serial(") + deviceFile_ + wxT("): can't write data, device is not open."));
		return false;
	}
	
	// Check status.
	if ( !(*device_) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): can't write data, ") + wxString::FromAscii(device_->getErrorMessage().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	// Try to write data.
	if ( !device_->transmit(data) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): writing failed, ") + wxString::FromAscii(device_->getErrorMessage().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	return true;
}



bool Serial::portIsConnected() const
{
	return device_ != NULL;
}
