/* $Id: main.h 400 2010-04-17 18:16:56Z oh2gve $
 *
 * Copyright 2010, 2011 Tapio Aaltonen
 *
 *
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// Own stuff.
#include "window.h"
#include "icon.h"
#include "controller.h"

// Wx stuff.
#include <wx/frame.h>
#include <wx/string.h>
#include <wx/log.h>




// Event table for events handled in this code file.
BEGIN_EVENT_TABLE(Window, wxFrame)
	EVT_MENU(ID_FileQuit, Window::onFileQuit)
	EVT_MENU(ID_HelpAbout, Window::onHelpAbout)
	EVT_CLOSE(Window::onClose)

	EVT_COMMAND(Controller::ID_fatalError, wxEVT_COMMAND_TEXT_UPDATED, Window::onControllerFailure)
	EVT_COMMAND(Controller::ID_newPacket, wxEVT_COMMAND_TEXT_UPDATED, Window::onControllerNewPacket)
END_EVENT_TABLE()


Window::Window(wxString const& title):
wxFrame(NULL, -1, title), icon_(NULL)
{
	// Set process icon.
	wxIcon myico = wxIcon(wxT("aprsg.ico"), wxBITMAP_TYPE_ICO);
	if ( myico.IsOk() )
	{
		SetIcon(myico);
	}

	// Put our icon in taskbar.
	icon_ = new Icon(this);
	if ( icon_ == NULL || !(*icon_) )
	{
		delete icon_;
		icon_ = NULL;
	}

	// Create status bar.
	CreateStatusBar();
	SetStatusText(wxT("Welcome!"));
}

Window::~Window()
{
	delete icon_;
}


void Window::onFileQuit(wxCommandEvent& event)
{
}

void Window::onHelpAbout(wxCommandEvent& event)
{
}

void Window::onClose(wxCloseEvent& event)
{
	Destroy();
}

void Window::onControllerFailure(wxCommandEvent& event)
{
	Close();
}

void Window::onControllerNewPacket(wxCommandEvent& event)
{
	SetStatusText(event.GetString());
}
